@@ -20,6 +20,6 @@
 
 #include "gnome-ppp.h"
 
-void open_account_window (Account * account);
+void open_account_window (Account * account, gboolean clone);
 
 #endif /* __account_window_h__ */
