@@ -51,31 +51,39 @@ typedef struct _Dial
   GtkWidget *connect_button_label;
   GtkWidget *connect;
   GtkWidget *debug_terminal_check_menu_item;
-
   GtkWidget *term_app;
   GtkWidget *term;
 
-  GtkWidget *input_dialog;
-
-  Account *account;
-  gint state;
-  gint timer_id;
+  GList     *active_list;
 } Dial;
 
 
+typedef struct _Active
+{
+  Dial      *dial;
+  Account   *account;
+  gint       state;
+  gint       pppd_pid;
+  gint       ppp_device;
+  GtkWidget *input_dialog;
+} Active;
+
+
+
 static void refresh_dial_windows();
 static void refresh_dial_app(Dial *dial);
 static void refresh_dial_account_list(Dial *dial);
+static void refresh_dial_button(Dial *dial);
 static gint close_dial_app(GtkWidget *widget, Dial *dial);
 
 
 /* app callbacks */
+static void account_clist_select_cb(GtkWidget *widget);
 static void destroy_dial_app_cb(GtkWidget *widget);
 static void connect_button_cb(GtkWidget *widget);
 
 
 /* dial engine callbacks */
-static gint timeout_cb(Dial *dial);
 static void connect_message_cb(Account         *account, 
 			       PPPMessageType   message,
 			       gchar           *text, 
@@ -99,6 +107,11 @@ static void free_dial(Dial *dial);
 static Dial *get_dial_context(GtkObject *object);
 static void set_dial_context(GtkObject *object, Dial *dial);
 static Account *get_selected_account(Dial *dial);
+static Active *get_active_account(Dial *dial, Account *account);
+static Active *malloc_active();
+static void free_active(Active *active);
+static Active *get_active_context(GtkObject *object);
+static void set_active_context(GtkObject *object, Active *active);
 
 
 /* terminal stuff */
@@ -120,9 +133,6 @@ static void input_dialog_cb(gchar *string, gpointer data);
 /* menus */
 static GnomeUIInfo file_menu[] = 
 {
-  GNOMEUIINFO_MENU_NEW_ITEM(N_("New Dialer"), NULL, menu_file_new_dialer_cb, NULL),
-  GNOMEUIINFO_SEPARATOR,
-  GNOMEUIINFO_MENU_CLOSE_ITEM(menu_file_exit_cb, NULL),
   GNOMEUIINFO_MENU_EXIT_ITEM(menu_file_exit_cb, NULL),
   GNOMEUIINFO_END
 };
@@ -147,6 +157,7 @@ static GnomeUIInfo account_menu[] =
 
 static GnomeUIInfo help_menu[] =
 {
+  GNOMEUIINFO_HELP ("gnome-ppp"),
   GNOMEUIINFO_MENU_ABOUT_ITEM(menu_help_about_cb, NULL),
   GNOMEUIINFO_END
 };
@@ -179,6 +190,11 @@ new_dial_app()
   Dial *dial;
   GtkWidget *vbox, *hbox;
 
+  gchar *titles[] = {
+    "Name", "Status", "PID", "Device"
+  };
+
+
   /* initalize account watcher to update the clist of all dial
    * application windows
    */
@@ -226,12 +242,24 @@ new_dial_app()
   gtk_widget_show(vbox);
 
   /* account clist */
-  dial->account_clist = gtk_clist_new(1);
+  dial->account_clist = gtk_clist_new_with_titles(4, titles);
   set_dial_context(GTK_OBJECT(dial->account_clist), dial);
   gtk_box_pack_start(GTK_BOX(vbox), dial->account_clist, TRUE, TRUE, 0);
 
+  gtk_clist_set_column_auto_resize(GTK_CLIST(dial->account_clist), 0, 1);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(dial->account_clist), 1, 1);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(dial->account_clist), 2, 1);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(dial->account_clist), 3, 1);
+
   gtk_clist_column_titles_passive(GTK_CLIST(dial->account_clist));
   gtk_clist_set_selection_mode(GTK_CLIST(dial->account_clist), GTK_SELECTION_BROWSE);
+
+  gtk_signal_connect(
+      GTK_OBJECT(dial->account_clist),
+      "select_row",
+      GTK_SIGNAL_FUNC(account_clist_select_cb),
+      NULL);
+
   gtk_widget_show(dial->account_clist);
 
   /* connect button */
@@ -278,17 +306,27 @@ refresh_dial_windows()
     {
       dial = (Dial *) list->data;
       list = list->next;
-      refresh_dial_account_list(dial);
+      refresh_dial_app(dial);
     } 
 }
 
 
+static void
+refresh_dial_app(Dial *dial)
+{
+  refresh_dial_account_list(dial);
+  refresh_dial_button(dial);
+}
+
+
 static void
 refresh_dial_account_list(Dial *dial)
 {
   GList *list;
+  Active *active;
   Account *saved_account, *account;
 
+
   /* save selected account */
   saved_account = get_selected_account(dial);
 
@@ -300,12 +338,47 @@ refresh_dial_account_list(Dial *dial)
   while (list)
     {
       gint row;
-      gchar *text[1];
+      gchar *text[4], pid_buff[32], device_buff[32];
 
       account = (Account *) list->data;
       list = list->next;
 
       text[0] = account->name->str;
+      text[1] = NULL;
+      text[2] = NULL;
+      text[3] = NULL;
+
+      /* active is the data structure associated with a actively used account */
+      active = get_active_account(dial, account);
+      if (active)
+	{
+	  switch (active->state)
+	    {
+	    case _IN_PROGRESS:
+	      text[1] = _("Dialing");
+	      break;
+	      
+	    case _CONNECTED:
+	      text[1] = _("Connected");
+	      break;
+	      
+	    default:
+	      break;
+	    }
+
+	  if (active->pppd_pid >= 0)
+	    {
+	      snprintf(pid_buff, 31, "%d", active->pppd_pid);
+	      text[2] = pid_buff;
+	    }
+
+	  if (active->ppp_device >= 0)
+	    {
+	      snprintf(device_buff, 31, "ppp%d", active->ppp_device);
+	      text[3] = device_buff;
+	    } 
+	}
+
       row = gtk_clist_append(GTK_CLIST(dial->account_clist), text);
       if (row < 0)
 	{
@@ -319,41 +392,52 @@ refresh_dial_account_list(Dial *dial)
 	  gtk_clist_select_row(GTK_CLIST(dial->account_clist), row, 0);
 	}
     }
+
   gtk_clist_thaw(GTK_CLIST(dial->account_clist));
 }
 
 
 static void
-refresh_dial_app(Dial *dial)
+refresh_dial_button(Dial *dial)
 {
-  /* set the active state of the connection button and the
-   * account-selection option menu based on the connection state
-   */
-  if (account_list())
-    {
-      switch (dial->state)
-	{
-	case _DISCONNECTED:
-	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Connect"));
-	  gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), TRUE);
-	  break;
+  Active *active;
 
-	case _IN_PROGRESS:
-	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Cancel"));
-	  gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), TRUE);
-	  break;
 
-	case _CONNECTED:
-	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Disconnect"));
-	  gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), TRUE);
-	  break;
-	}
-    }
-  else
+  /* disable button if there are no accounts */
+  if (!account_list())
     {
       gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Connect"));
       gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), FALSE);
+      return;
     }
+
+  active = get_active_account(dial, get_selected_account(dial));
+  if (!active)
+    {
+      gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Connect"));
+    }
+  else
+    {
+      /* set the active state of the connection button and the
+       * account-selection option menu based on the connection state
+       */
+      switch (active->state)
+	{
+	case _IN_PROGRESS:
+	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Cancel"));
+	  break;
+	  
+	case _CONNECTED:
+	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Disconnect"));
+	  break;
+
+	default:
+	  g_error("refresh_dial_button(): unknown state");
+	  break;
+	}
+    }
+
+  gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), TRUE);
 }
 
 
@@ -363,7 +447,7 @@ close_dial_app(GtkWidget *widget, Dial *dial)
   dial = get_dial_context(GTK_OBJECT(widget));
 
   /* don't close if the connection is active */
-  if (dial->state != _DISCONNECTED)
+  if (dial->active_list)
     {
       gnome_error_dialog(_("You must disconnect before closing."));
       return TRUE;
@@ -393,196 +477,189 @@ destroy_dial_app_cb(GtkWidget *widget)
 
 
 static void
-connect_button_cb(GtkWidget *widget)
+account_clist_select_cb(GtkWidget *widget)
 {
-  Dial *dial;
-
-  dial = get_dial_context(GTK_OBJECT(widget));
-
-  switch (dial->state)
-    {
-    case _DISCONNECTED:
-      g_assert(dial->account == NULL);
-      g_assert(GTK_CLIST(dial->account_clist)->selection);
-
-      /* get selected account */
-      if (!(dial->account = get_selected_account(dial)))
-	{
-	  g_error(_("connect_button_cb(): no selected account"));
-	}
-
-      /* locked accounts are already in use */
-      if (!account_is_locked(dial->account))
-	{
-	  account_lock(dial->account);
-
-	  /* start the connection; if the start is successful, then
-	   * add the timeout callback to run the state engine
-	   */
-	  if (connect_start(dial->account, connect_message_cb, dial))
-	    {
-	      dial->timer_id = 
-		gtk_timeout_add(500, (GtkFunction) timeout_cb, dial);
-	    }
-	}
-      else
-	{
-	  gnome_error_dialog(_("This account is in use."));
-	  dial->account = NULL; 
-	}
-    break;
-
-    case _IN_PROGRESS:
-    case _CONNECTED:
-      g_assert(dial->account);
-      connect_stop(dial->account);
-    break;
-    }
+  Dial *dial = get_dial_context(GTK_OBJECT(widget));
+  refresh_dial_button(dial);
 }
 
 
-static gint
-timeout_cb(Dial *dial)
+static void
+connect_button_cb(GtkWidget *widget)
 {
-  if (connect_engine_iteration(dial->account))
+  Dial *dial;
+  Account *account;
+  Active *active;
+
+
+  dial = get_dial_context(GTK_OBJECT(widget));
+  account = get_selected_account(dial);
+  if (!account)
     {
-      return 1;
+      g_error("connect_button_cb(): account == NULL");
     }
-  else
+
+  active = get_active_account(dial, account);
+  if (!active)
     {
-      /* blank out the timeout id and return 0 shutting
-       * down the gtk_timeout
-       */
-      dial->timer_id = -1;
-      return 0;
+      active = malloc_active();
+      active->dial = dial;
+      active->account = account;
+      dial->active_list = g_list_append(dial->active_list, active);
+    }
+
+  switch (active->state)
+    {
+    case _DISCONNECTED:
+      if (connect_start(active->account, connect_message_cb, active))
+	{
+	  active->pppd_pid = connect_pid(active->account);
+	  refresh_dial_account_list(active->dial);
+	}
+    break;
+
+
+    case _IN_PROGRESS:
+    case _CONNECTED:
+      connect_stop(active->account);
+    break;
     }
 }
 
 
 /* message callback from connect */
 static void
-connect_message_cb(Account         *account, 
-		   PPPMessageType   message, 
-		   gchar           *text, 
-		   gpointer         data)
+disconnected_cb(Active *active, gchar *text)
 {
-  gboolean update_statusbar;
-  gint saved_state;
-  Dial *dial;
+  Dial *dial = active->dial;
 
-  dial = (Dial *) data;
-  update_statusbar = FALSE;
-  saved_state = dial->state;
 
-  /* sanity checks */
-  if (dial->account != account)
+  /* get rid of any remaining input dialog */
+  if (active->input_dialog)
     {
-      g_error("connect_message_cb(): account != dial->account");
+      gtk_widget_destroy(active->input_dialog);
+    }
+  
+  /* remove the active account from the dial app's active list */
+  dial->active_list = g_list_remove(dial->active_list, active);
+  free_active(active);
+
+  refresh_dial_app(dial);
+}
+
+
+static void
+in_progress_cb(Active *active, gchar *text)
+{
+  active->state = _IN_PROGRESS;
+
+  if (text)
+    {
+      gnome_appbar_set_status(GNOME_APPBAR(active->dial->appbar), _(text));
+    }
+  else
+    {
+      gnome_appbar_set_status(GNOME_APPBAR(active->dial->appbar), "");
     }
 
-  /* map messages from the connect engine to internal
-   * states of the dial window
-   */
+  refresh_dial_app(active->dial);
+}
+
+
+static void
+connected_cb(Active *active, gchar *text)
+{
+  if (active->state != _CONNECTED)
+    {
+      active->state = _CONNECTED;
+      active->ppp_device = connect_device(active->account);
+
+      if (text)
+	{
+	  gnome_appbar_set_status(GNOME_APPBAR(active->dial->appbar), _(text));
+	}
+      else
+	{
+	  gnome_appbar_set_status(GNOME_APPBAR(active->dial->appbar), "");
+	}
+
+      refresh_dial_app(active->dial);
+    }
+}
+
+
+static void
+error_cb(Active *active, gchar *text)
+{
+  gnome_error_dialog(_(text));
+}
+
+
+debug_cb(Active *active, gchar *text)
+{
+  terminal_write(active->dial, _(text));
+}
+
+
+static void
+prompt_input_cb(Active *active, gchar *text)
+{
+  active->input_dialog = 
+    gnome_request_dialog(TRUE, _(text), "", 100, (GnomeStringCallback) input_dialog_cb, 
+			 active, NULL);
+      
+  set_active_context(GTK_OBJECT(active->input_dialog), active);
+
+  gtk_signal_connect(
+      GTK_OBJECT(active->input_dialog),
+      "destroy",
+      GTK_SIGNAL_FUNC(input_dialog_destroy_cb),
+      NULL);
+}
+
+
+static void
+connect_message_cb(Account *account, PPPMessageType message, gchar *text, gpointer data)
+{
+  Active *active = (Active *) data;
+  Dial *dial = active->dial;
+
+
+  /* map messages from the connect engine to internal states of the dial window */
   switch (message)
     {
     case PPP_DISCONNECTED:
-      dial->state = _DISCONNECTED;
-      update_statusbar = TRUE;
+      disconnected_cb(active, text);
       break;
 
     case PPP_IN_PROGRESS:
-      dial->state = _IN_PROGRESS;
-      update_statusbar = TRUE;
+      in_progress_cb(active, text);
       break;
 
     case PPP_CONNECTED:
-      dial->state = _CONNECTED;
-      update_statusbar = TRUE;
+      connected_cb(active, text);
       break;
 
     case PPP_ERROR:
-      gnome_error_dialog(_(text));
+      error_cb(active, text);
       break;
 
     case PPP_DEBUG:
-      terminal_write(dial, _(text));
-      return;
+      debug_cb(active, text);
       break;
 
     case PPP_PROMPT_INPUT:
-      if (dial->input_dialog)
-	{
-	  g_error("connect_message_cb(): dial->user_input_dialog != NULL");
-	}
-
-      dial->input_dialog = gnome_request_dialog(
-	                       TRUE,
-			       _(text),
-			       "",
-			       100,
-			       (GnomeStringCallback) input_dialog_cb,
-			       dial,
-			       NULL);
-      
-      set_dial_context(GTK_OBJECT(dial->input_dialog), dial);
-
-      gtk_signal_connect(
-          GTK_OBJECT(dial->input_dialog),
-	  "destroy",
-	  GTK_SIGNAL_FUNC(input_dialog_destroy_cb),
-	  NULL);
-
+      prompt_input_cb(active, text);
       break;
 
     default:
       g_error("connect_message_cb(): unknown callback message");
       break;
     }
-
-  /* display a message if there is one */
-  if (update_statusbar)
-    {
-      if (text)
-	{
-	  gnome_appbar_set_status(GNOME_APPBAR(dial->appbar), _(text));
-	}
-      else
-	{
-	  gnome_appbar_set_status(GNOME_APPBAR(dial->appbar), "");
-	}
-    }
-
-  /* if the state is disconnected, then unlock the account */
-  if (dial->state == _DISCONNECTED)
-    {
-      /* get rid of any remaining input dialog */
-      if (dial->input_dialog)
-	{
-	  gtk_widget_destroy(dial->input_dialog);
-	}
-
-      account_unlock(account);
-      dial->account = NULL;
-    }
-
-
-  /* refresh the dial window if the state changed */
-  if (dial->state != saved_state)
-    {
-      refresh_dial_app(dial);
-    }
 }
 
 
 /* menu callbacks need to be different */
-static void
-menu_file_new_dialer_cb(GtkWidget *widget, Dial *dial)
-{
-  g_assert(dial != NULL);
-  new_dial_app();
-}
-
-
 static void
 menu_file_exit_cb(GtkWidget *widget, Dial *dial)
 {
@@ -697,17 +774,10 @@ static Dial*
 malloc_dial()
 {
   Dial *dial = g_malloc(sizeof(Dial));
-  dial->account = NULL;
-  dial->state = _DISCONNECTED;
-  dial->timer_id = -1;
-
   dial->term_app = NULL;
   dial->term = NULL;
-
-  dial->input_dialog = NULL;
-
-  __dial_app_list = g_list_append(__dial_app_list, dial);
-  
+  dial->active_list = NULL;
+  __dial_app_list = g_list_append(__dial_app_list, dial);  
   return dial;
 }
 
@@ -733,6 +803,7 @@ set_dial_context(GtkObject *object, Dial *dial)
   gtk_object_set_data(object, "dial_context", dial);
 }
 
+
 static Account *
 get_selected_account(Dial *dial)
 {
@@ -750,6 +821,63 @@ get_selected_account(Dial *dial)
 }
 
 
+static Active*
+get_active_account(Dial *dial, Account *account)
+{
+  Active *active;
+  GList *list = dial->active_list;
+  
+  while (list)
+    {
+      active = list->data;
+      list = list->next;
+      
+      if (active->account == account)
+	{
+	  return active;
+	}
+    }
+
+  return NULL;
+}
+
+
+static Active*
+malloc_active()
+{
+  Active *active = g_malloc(sizeof(Active));
+  active->dial = NULL;
+  active->account = NULL;
+  active->state = _DISCONNECTED;
+  active->pppd_pid = -1;
+  active->ppp_device = -1;
+  active->input_dialog = NULL;
+  return active;
+}
+
+
+static void
+free_active(Active *active) 
+{
+  g_free(active);
+}
+
+
+static Active* 
+get_active_context(GtkObject *object)
+{
+  return gtk_object_get_data(object, "active_context");
+}
+
+
+static void
+set_active_context(GtkObject *object, Active *active)
+{
+  gtk_object_set_data(object, "active_context", active);
+}
+
+
+
 /*** terminal ***/
 static void
 new_terminal_app(Dial *dial)
@@ -830,46 +958,6 @@ static void
 terminal_write(Dial *dial, gchar *buff)
 {
   zvt_term_feed(ZVT_TERM(dial->term), buff, strlen(buff));
-#if 0
-  int state;
-
-  state = 0;
-  while (*buff != '\0')
-    {
-      switch (state)
-	{
-	case 0:
-	  if (*buff == '\r')
-	    {
-	      state = 1;
-	      zvt_term_feed(ZVT_TERM(dial->term), "\r\n", 2);
-	    }
-	  else
-	    {
-	      zvt_term_feed(ZVT_TERM(dial->term), buff, 1);
-	    }
-	  break;
-
-	case 1:
-	  if (*buff == '\r')
-	    {
-	      zvt_term_feed(ZVT_TERM(dial->term), "\r\n", 2);
-	    }
-	  else if (*buff == '\n')
-	    {
-	      state = 0;
-	    }
-	  else
-	    {
-	      state = 0;
-	      zvt_term_feed(ZVT_TERM(dial->term), buff, 1);
-	    }
-	  break;
-	}
-
-      buff++;
-    }
-#endif
 }
 
 
@@ -935,21 +1023,14 @@ terminal_menu_close_cb(GtkWidget *widget, Dial *dial)
 static void
 input_dialog_destroy_cb(GtkWidget *widget)
 {
-  Dial *dial;
-
-  dial = get_dial_context(GTK_OBJECT(widget));
-  dial->input_dialog = NULL;
+  Active *active = get_active_context(GTK_OBJECT(widget));
+  active->input_dialog = NULL;
 }
 
 
-
 static void 
 input_dialog_cb(gchar *string, gpointer data)
 {
-  Dial *dial;
-
-  dial = (Dial *) data;
-  g_assert(dial->account != NULL);
-
-  connect_send(dial->account, string, TRUE);
+  Active *active = (Active *) data;
+  connect_send(active->account, string, TRUE);
 }
