@@ -18,8 +18,7 @@
 #ifndef __dial_window_h__
 #define __dial_window_h__
 
-void open_dial_window ();
-void refresh_dial_window ();
+void new_dial_app();
 
 #endif /* __dial_window_h__ */
 
