@@ -18,12 +18,25 @@
 #ifndef __constants_h__
 #define __constants_h__
 
+
 #define GNOME_PPP_NAME             "gnome-ppp"
-#define GPPP_BUTTON_HEIGHT          30
-#define GPPP_BUTTON_WIDTH           70
-#define CONFIGURE_WINDOW_WIDTH     275
-#define CONFIGURE_WINDOW_HEIGHT    325
-#define ACCOUNT_WINDOW_WIDTH       350
-#define ACCOUNT_WINDOW_HEIGHT      400
+#define GNOME_PPP_TITLE            "Gnome PPP"
+#define GNOME_PPP_VERSION          "0.99.0"
+
+/* you can't change author/copyright strings in the
+ * source code; but you can change them in the about
+ * box of the executeable if you wish (or disable the about box)
+ */
+#define GNOME_PPP_AUTHOR           "Jay Painter"
+#define GNOME_COPYRIGHT            "(C) 1998,1999 Jay Painter"
+
+/* look/feel constants */
+#define BUTTON_HEIGHT          30
+#define BUTTON_WIDTH           70
+#define TABLE_ROW_SPACINGS     10
+#define TABLE_COL_SPACINGS      5
+#define BOX_SPACING            10
+#define CONTAINER_BORDER        5
+
 
 #endif /* __constants_h__ */
