@@ -36,8 +36,8 @@
 #endif
 
 
-#ifndef CMSG_DATA /* Linux libc5 */
-/* Ancillary data object manipulation macros.  */
+/* linux libc5 ancillary data object manipulation macros */
+#ifndef CMSG_DATA
 #if !defined __STRICT_ANSI__ && defined __GNUC__ && __GNUC__ >= 2
 # define CMSG_DATA(cmsg) ((cmsg)->cmsg_data)
 #else
@@ -47,11 +47,15 @@
 
 #define CONTROLLEN (sizeof(struct cmsghdr) + sizeof(int))
 
-/* define a random byte of data to send 
- * along with the file descriptor
- */
+/* define a random byte of data to send along with the file descriptor */
 #define RANDOM_BYTE "Z"
 
+/* max number of connection retries to the domain socket and the pause
+ * time between connection attempts
+ */
+#define MAX_CONNECT_RETRIES 5
+#define SEC_BETWEEN_RETRIES 2
+
 
 /* prototypes */
 int recieve_fd(int fd, int *recvfd);
@@ -61,17 +65,22 @@ int send_fd(int fd, int sendfd);
 int
 main(int argc, char *argv[])
 {
-  int fd, len;
+  int fd, len, conn_stat, retry = 0;
   struct timeval tv;
   struct sockaddr_un name;
   fd_set rset;
 
+
   /* gnome-ppp-chat only takes one argument */
   if (argc != 2)
-    exit(1);
+    {
+      exit(1);
+    }
 
   if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
-    exit(1);
+    {
+      exit(1);
+    }
 
   /* set address/path of the server */
   memset(&name, 0, sizeof(struct sockaddr_un));
@@ -79,16 +88,34 @@ main(int argc, char *argv[])
   strcpy(name.sun_path, argv[1]);
   len = sizeof(name.sun_family) + strlen(name.sun_path);
 
-  /* connect to the server */
-  if (connect(fd, (struct sockaddr *) &name, len) < 0)
-    exit(1);
+  /* connect to the server, wait, if necessary */
+  while (1)
+    {
+      conn_stat = connect(fd, (struct sockaddr *) &name, len);
+      if (conn_stat < 0 && errno == ECONNREFUSED)
+	{
+	  /* exit after the maximum tries */
+	  if (++retry > MAX_CONNECT_RETRIES)
+	    {
+	      exit(1);
+	    }
+
+	  /* pause before tring to connect */
+	  sleep(SEC_BETWEEN_RETRIES);
+	  continue;
+	}
+
+      break;
+    }
 
   /* now we want to pass the file descriptor of the modem
    * to the server; we can either pass 0 or 1 since they
    * are both the same
    */
   if (!send_fd(fd, 0))
-    exit(1);
+    {
+      exit(1);
+    }
 
   /* the server will send a byte of data when finished */
   FD_ZERO(&rset);
@@ -110,8 +137,11 @@ send_fd(int fd, int sendfd)
   struct iovec iov[1];
   struct cmsghdr *cmptr;
 
+
   if (!(cmptr = malloc(CONTROLLEN)))
-    exit(1);
+    {
+      exit(1);
+    }
 
   iov[0].iov_base     = RANDOM_BYTE;
   iov[0].iov_len      = 1;
@@ -131,8 +161,9 @@ send_fd(int fd, int sendfd)
   *((int *) CMSG_DATA(cmptr)) = sendfd;
 
   if (sendmsg(fd, &msg, 0) <= 0)
-    goto error;
-
+    {
+      goto error;
+    }
 
   free(cmptr);
   return 1;
