@@ -25,29 +25,32 @@
 #include "modem.h"
 #include "pppd.h"
 #include "program.h"
+#include "msg.h"
 
 
 /* XXX: this is lame. -JMP */
 /* various timeouts in seconds for operations */
-#define HANGUP_TIMEOUT        2
+#define HANGUP_TIMEOUT        3
 #define GET_TIMEOUT           5
 #define DETECT_TIMEOUT        5
 #define DIAL_TIMEOUT         60
+#define REDIAL_PAUSE_TIMEOUT  5
 #define SCRIPT_TIMEOUT       30
 #define PPPD_TIMEOUT          5
 
 #define FIRST_STATE           0
 #define MAX_HANGUP_ATTEMPTS   3
+#define NEWLINE               "\r"
 
 
 /* a couple of macros to make the code more readable */
 #define SET_TIMEOUT(t, s) ((t) = time(NULL) + (s))
 #define TIMEOUT(t)        ((t) < time(NULL))
 #define NEXT_STATE(s)     ((s)++)
-#define CALLBACK(c, t, m) ((*(c)->func)((c)->account, (t), (m), ((c)->data)))
+#define CALLBACK(c, t, m) ((*(c)->func)((c)->orig_account, (t), (m), ((c)->data)))
 
 
-/* master state */
+/* master states */
 enum
 {
   _GET_MODEM,
@@ -59,40 +62,33 @@ enum
 };
 
 
-/* message indexes for callback messages */
-enum
+/* modem response strings */
+#define NO_DIAL_TONE_STRING   "NO DIAL TONE"NEWLINE
+#define NO_CARRIER_STRING     "NO CARRIER"NEWLINE
+#define BUSY_STRING           "BUSY"NEWLINE
+#define NO_ANSWER_STRING      "NO ANSWER"NEWLINE
+#define CONNECT_STRING        "CONNECT"
+
+static gchar *__modem_responses[] =
 {
-  _MSG_NO_PPPD_EXEC_PERMISSION,
-  _MSG_MODEM_LOCKED,
-  _MSG_PPPD_FORK,
-  _MSG_CLOSING,
-  _MSG_MODEM_FD_PASS_TIMEOUT,
-  _MSG_MODEM_HANUP_FAILED,
-  _MSG_MODEM_DETECT,
-  _MSG_MODEM_DETECT_FAILED,
-  _MSG_MAX_REDIAL,
-  _MSG_NO_PHONE_NUMBER,
-  _MSG_DIALING,
-  _MSG_DIAL_TIMEOUT,
-  _MSG_NO_DIALTONE,
-  _MSG_BUSY,
-  _MSG_CONNECTED,
-  _MSG_NO_CARRIOR,
-  _MSG_RUNNING_SCRIPT,
-  _MSG_SCRIPT_COMPLETE,
-  _MSG_PPP_CONNECTED,
-  _MSG_PPP_FAIL,
-  _MSG_PPP_UNKNOWN_DEATH,
-  _MSG_LAST_MESSAGE
+  NO_DIAL_TONE_STRING,
+  NO_CARRIER_STRING,
+  BUSY_STRING,
+  NO_ANSWER_STRING,
+  CONNECT_STRING,
+  NULL
 };
 
 
+
 typedef struct _Connect
 {
-  Account *account;
+  Account *orig_account, *account;
 
   ConnectCBFunc func;
   gpointer data;
+  gint pppd_pid;
+  gint ppp_device;
 
   Modem *modem;
   GString *modem_buffer;
@@ -130,9 +126,13 @@ static Connect*
 malloc_connect()
 {
   Connect *connect = g_malloc(sizeof(Connect));
+
+  connect->orig_account = NULL;
   connect->account = NULL;
   connect->func = NULL;
   connect->data = NULL;
+  connect->pppd_pid = -1;
+  connect->ppp_device = -1;
   connect->modem = NULL;
   connect->modem_buffer = g_string_new("");
   connect->timeout = 0;
@@ -168,25 +168,18 @@ free_connect(Connect *connect)
 {
   g_assert(connect != NULL);
 
+  /* free the cloned account */
+  if (connect->account)
+    {
+      account_free(connect->account);
+    }
+
   g_string_free(connect->modem_buffer, TRUE);
   g_string_free(connect->send_buffer, TRUE);
   g_free(connect);
 }
 
 
-/* list of all active connections  */
-static gchar           *__messages[_MSG_LAST_MESSAGE];
-static GHashTable      *__connect_hash = NULL;
-static gchar           *__modem_responses[] =
-{
-  "NO DIALTONE\r",
-  "NO CARRIER\r",
-  "BUSY\r",
-  "CONNECT",
-  NULL
-};
-
-
 /* state machine for the entire connection sequence */
 static gint state_engine_iteration(Connect *connect);
 static void state_get_modem(Connect *connect);
@@ -208,6 +201,12 @@ static gboolean search_modem_buffer(Connect *connect, gchar *find);
 static gchar *search_modem_buffer_list(Connect *connect, gchar *find_list[]);
 
 
+
+
+/* list of all active connections  */
+static GHashTable      *__connect_hash = NULL;
+
+
 /* initalizer registered with init.c */
 void
 __init_connect()
@@ -218,67 +217,6 @@ __init_connect()
     {
       done_init = TRUE;
       __connect_hash = g_hash_table_new(g_direct_hash, NULL);
-
-      /* initalize messages */
-      __messages[_MSG_NO_PPPD_EXEC_PERMISSION] =
-	_("You don't have permission to execute pppd.");
-
-      __messages[_MSG_MODEM_LOCKED] =
-	_("The modem is in use by another program.");
-
-      __messages[_MSG_PPPD_FORK] =
-	_("Running pppd.");
-
-      __messages[_MSG_CLOSING] =
-	_("Closing connection.");
-
-      __messages[_MSG_MODEM_FD_PASS_TIMEOUT] =
-	_("The pppd program failed to run gnome-ppp-chat.");
-
-      __messages[_MSG_MODEM_HANUP_FAILED] =
-	_("The modem could not be hung up.");
-
-      __messages[_MSG_MODEM_DETECT] =
-	_("Initalizing modem.");
-
-      __messages[_MSG_MODEM_DETECT_FAILED] =
-	_("Your modem is not responding to initalization.");
-
-      __messages[_MSG_MAX_REDIAL] = 
-	_("No answer after maximum number of re-dial attempts.");
-
-      __messages[_MSG_NO_PHONE_NUMBER] =
-	_("No phone numbers specified for this account.");
-
-      __messages[_MSG_DIALING] =
-	_("Dialing.");
-
-      __messages[_MSG_NO_DIALTONE] =
-	_("No dialtone.");
-
-      __messages[_MSG_BUSY] =
-	_("Number Busy.");
-
-      __messages[_MSG_CONNECTED] =
-	_("Connected.");
-      
-      __messages[_MSG_NO_CARRIOR] = 
-	_("The modem lost connection with remote host.");
-
-      __messages[_MSG_RUNNING_SCRIPT] =
-	_("Running Script.");
-
-      __messages[_MSG_SCRIPT_COMPLETE] =
-	_("Script Complete.");
-
-      __messages[_MSG_PPP_CONNECTED] =
-	_("PPP connection established.");
-
-      __messages[_MSG_PPP_FAIL] =
-	_("Failed to establish a PPP connection.");
-
-      __messages[_MSG_PPP_UNKNOWN_DEATH] =
-	_("The pppd daemon died unexpectedly.");
     }
 }
 
@@ -293,22 +231,36 @@ connect_start(Account *account, ConnectCBFunc func, gpointer data)
   /* don't start a connection on a account that's already in use */
   if (g_hash_table_lookup(__connect_hash, account))
     {
-      return;
+      CALLBACK(connect, PPP_ERROR, msg(MSG_IN_USE));
+      CALLBACK(connect, PPP_DISCONNECTED, NULL);
+      return 0;
     }
 
-  /* create connect info struct */
+  /* create connect info struct, reference this structure by the origional
+   * account pointer, but create a duplicate of the account for the diration
+   * of this connection so the origional account doesn't have to be locked
+   * aginst editing
+   */
   connect = malloc_connect();
-  connect->account = account;
   connect->func = func;
   connect->data = data;
+  connect->orig_account = account;
+  connect->account = account_clone(account);
+
+
 
   /* add to the hash */
   g_hash_table_insert(__connect_hash, account, connect);
+  CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_BEGIN));
 
   /* for pppd -- this forks pppd with the gnome-ppp-chat */
   if (pppd_exec(connect->account, pppd_die_cb, connect))
     {
-      CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_PPPD_FORK]);
+      connect->pppd_pid = pppd_pid(connect->account);
+
+      g_timeout_add(1000, (GSourceFunc) state_engine_iteration, connect);
+
+      CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_PPPD_FORK));
       CALLBACK(connect, PPP_DEBUG, _("\r\n---NEW CONNECTION---\r\n"));
     }
   else
@@ -316,11 +268,11 @@ connect_start(Account *account, ConnectCBFunc func, gpointer data)
       switch (pppd_errno)
 	{
 	case PPPD_NO_EXEC_PERMISSION:
-	  CALLBACK(connect, PPP_ERROR, __messages[_MSG_NO_PPPD_EXEC_PERMISSION]);
+	  CALLBACK(connect, PPP_ERROR, msg(MSG_NO_PPPD_EXEC_PERMISSION));
 	  goto error;
 
 	case PPPD_MODEM_LOCKED:
-	  CALLBACK(connect, PPP_ERROR, __messages[_MSG_MODEM_LOCKED]);
+	  CALLBACK(connect, PPP_ERROR, msg(MSG_MODEM_LOCKED));
 	  goto error;
 
 	default:
@@ -329,34 +281,17 @@ connect_start(Account *account, ConnectCBFunc func, gpointer data)
 	}
     }
 
-  return pppd_pid(connect->account);
+  return 1;
 
  error:
   /* remove connection from hash */
   CALLBACK(connect, PPP_DISCONNECTED, NULL);
-  g_hash_table_remove(__connect_hash, connect->account);
+  g_hash_table_remove(__connect_hash, connect->orig_account);
   free_connect(connect);
   return 0;
 }
 
 
-gint
-connect_engine_iteration(Account *account)
-{
-  Connect *connect;
-  
-  g_assert(account != NULL);
-
-  connect = g_hash_table_lookup(__connect_hash, account);
-  if (!connect)
-    {
-      return;
-    }
-
-  return state_engine_iteration(connect);
-}
-
-
 void
 connect_stop(Account *account)
 {
@@ -375,11 +310,11 @@ connect_stop(Account *account)
    */
   if (connect->state == _CONNECTED)
     {
-      CALLBACK(connect, PPP_CONNECTED, __messages[_MSG_CLOSING]);
+      CALLBACK(connect, PPP_CONNECTED, msg(MSG_CLOSING));
     }
   else
     {
-      CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_CLOSING]);
+      CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_CLOSING));
     }
 
   connect->close = TRUE;
@@ -404,7 +339,7 @@ connect_send(Account *account, gchar *text, gboolean append_cr)
   
   if (append_cr)
     {
-      g_string_sprintf(connect->send_buffer, "%s\r\n", text);
+      g_string_sprintf(connect->send_buffer, "%s"NEWLINE, text);
     }
   else
     {
@@ -413,6 +348,40 @@ connect_send(Account *account, gchar *text, gboolean append_cr)
 }
 
 
+gint
+connect_pid(Account *account)
+{  
+  Connect *connect;
+
+  g_assert(account != NULL);
+  
+  connect = g_hash_table_lookup(__connect_hash, account);
+  if (!connect)
+    {
+      return;
+    }
+  
+  return connect->pppd_pid;
+}
+
+
+gint
+connect_device(Account *account)
+{
+  Connect *connect;
+
+  g_assert(account != NULL);
+  
+  connect = g_hash_table_lookup(__connect_hash, account);
+  if (!connect)
+    {
+      return;
+    }
+  
+  return connect->ppp_device;
+}
+
+
 
 /* this is the main state machine for the ENTIRE connection cycle */
 static gint
@@ -469,7 +438,7 @@ state_engine_iteration(Connect *connect)
 	{
 	  if (!connect->user_closed)
 	    {
-	      CALLBACK(connect, PPP_ERROR, __messages[_MSG_PPP_UNKNOWN_DEATH]);
+	      CALLBACK(connect, PPP_ERROR, msg(MSG_PPP_UNKNOWN_DEATH));
 	    }
 
 	  /* run the disconnection program */
@@ -529,7 +498,7 @@ state_get_modem(Connect *connect)
 	    case PPPD_PENDING:
 	      if (TIMEOUT(connect->timeout))
 		{
-		  CALLBACK(connect, PPP_ERROR, __messages[_MSG_MODEM_FD_PASS_TIMEOUT]);
+		  CALLBACK(connect, PPP_ERROR, msg(MSG_MODEM_FD_PASS_TIMEOUT));
 		  connect->close = TRUE;
 		  connect->user_closed = TRUE;
 		  goto error;
@@ -554,7 +523,6 @@ state_get_modem(Connect *connect)
 static void
 state_detect_modem(Connect *connect)
 {
-  gint index;
   GString *gstr;
 
   enum
@@ -583,7 +551,7 @@ state_detect_modem(Connect *connect)
 	    }
 	  else
 	    {
-	      CALLBACK(connect, PPP_ERROR, __messages[_MSG_MODEM_HANUP_FAILED]);
+	      CALLBACK(connect, PPP_ERROR, msg(MSG_MODEM_HANUP_FAILED));
 	      connect->close = TRUE;
 	      connect->user_closed = TRUE;
 	    }
@@ -616,9 +584,9 @@ state_detect_modem(Connect *connect)
 
 
     case __AT_INIT:
-      CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_MODEM_DETECT]);
+      CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_MODEM_DETECT));
 
-      g_string_sprintf(gstr, "%s\r", connect->account->modem_init->str);
+      g_string_sprintf(gstr, "%s"NEWLINE, connect->account->modem_init->str);
       modem_write(connect->modem, gstr->str, gstr->len);
 
       SET_TIMEOUT(connect->timeout, DETECT_TIMEOUT);
@@ -627,12 +595,8 @@ state_detect_modem(Connect *connect)
       
 
     case __OK:
-      index = g_string_find(connect->modem_buffer, "OK\r", TRUE);
-
-      /* modem deteted -- flush the modem buffers and go to the
-       * next stage of dialup
-       */
-      if (search_modem_buffer(connect, "OK\r"))
+      /* check for modem 'OK' response from initalization */
+      if (search_modem_buffer(connect, "OK"NEWLINE))
 	{
 	  NEXT_STATE(connect->state);
 	}
@@ -643,7 +607,7 @@ state_detect_modem(Connect *connect)
 	   */
 	  if (TIMEOUT(connect->timeout))
 	    {
-	      CALLBACK(connect, PPP_ERROR, __messages[_MSG_MODEM_DETECT_FAILED]);
+	      CALLBACK(connect, PPP_ERROR, msg(MSG_MODEM_DETECT_FAILED));
 	      connect->close = TRUE;
 	      connect->user_closed = TRUE;
 	    }
@@ -660,16 +624,16 @@ static void
 state_dial(Connect *connect)
 {
   gchar *result;
-  GString *gstr;
-  GString *phone_number;
+  GString *gstr = g_string_new(""), *phone_number;
 
   enum
   {
     __CALL,
-    __ANSWER
+    __ANSWER,
+    __START_REDIAL_PAUSE,
+    __END_REDIAL_PAUSE
   };
 
-  gstr = g_string_new("");
 
   switch (connect->dial_state)
     {
@@ -677,7 +641,7 @@ state_dial(Connect *connect)
       /* check for maximum # of redials */
       if (connect->dial_attempts >= connect->account->redial_attempts)
 	{
-	  CALLBACK(connect, PPP_ERROR, __messages[_MSG_MAX_REDIAL]);
+	  CALLBACK(connect, PPP_ERROR, msg(MSG_MAX_REDIAL));
 	  connect->close = TRUE;
 	  connect->user_closed = TRUE;
 	  goto cleanup;
@@ -686,7 +650,7 @@ state_dial(Connect *connect)
       /* the account HAS to have at least one phone number to dial */
       if (g_list_length(connect->account->phone_list) == 0)
 	{
-	  CALLBACK(connect, PPP_ERROR, __messages[_MSG_NO_PHONE_NUMBER]);
+	  CALLBACK(connect, PPP_ERROR, msg(MSG_NO_PHONE_NUMBER));
 	  connect->close = TRUE;
 	  connect->user_closed = TRUE;
 	  goto cleanup;
@@ -706,7 +670,7 @@ state_dial(Connect *connect)
       phone_number = (GString *) connect->phone_number_node->data;
       if (phone_number)
 	{
-	  CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_DIALING]);
+	  CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_DIALING));
 
 	  g_string_sprintf(gstr, "ATDT%s\r", phone_number->str);
 	  modem_write(connect->modem, gstr->str, gstr->len);
@@ -725,7 +689,7 @@ state_dial(Connect *connect)
       if (!result && TIMEOUT(connect->timeout))
 	{
 	  g_string_assign(connect->modem_buffer, "");
-	  connect->dial_state = __CALL;
+	  connect->dial_state = __START_REDIAL_PAUSE;
 	  goto cleanup;
 	}
 
@@ -736,38 +700,61 @@ state_dial(Connect *connect)
 	}
 
       /* no dialtone */
-      if (strcmp(result, "NO DIALTONE\r") == 0)
+      if (strcmp(result, NO_DIAL_TONE_STRING) == 0)
 	{
-	  CALLBACK(connect, PPP_ERROR, __messages[_MSG_NO_DIALTONE]);
+	  CALLBACK(connect, PPP_ERROR, msg(MSG_NO_DIALTONE));
 	  connect->close = TRUE;
 	  connect->user_closed = TRUE;
 	  goto cleanup;
 	}
 
       /* no carrior */
-      if (strcmp(result, "NO CARRIER\r") == 0)
+      if (strcmp(result, NO_CARRIER_STRING) == 0)
 	{
-	  CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_BUSY]);
-	  connect->dial_state = __CALL;
+	  CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_NO_CONNECT));
+	  connect->dial_state = __START_REDIAL_PAUSE;
 	  goto cleanup;
 	}
 
       /* busy */
-      if (strcmp(result, "BUSY\r") == 0)
+      if (strcmp(result, BUSY_STRING) == 0)
 	{
-	  CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_BUSY]);
-	  connect->dial_state = __CALL;
+	  CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_BUSY));
+	  connect->dial_state = __START_REDIAL_PAUSE;
 	  goto cleanup;
 	}
-
-      /* we've connected */
-      if (strcmp(result, "CONNECT") == 0)
+ 
+      /* ring, but no answer */
+      if (strcmp(result, NO_ANSWER_STRING) == 0)
 	{
-	  CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_CONNECTED]);
+	  CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_NO_ANSWER));
+	  connect->dial_state = connect->dial_state = __START_REDIAL_PAUSE;
+	  goto cleanup;
+	}
+      
+      /* we've connected */
+      if (strcmp(result, CONNECT_STRING) == 0)
+	{
+	  CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_CONNECTED));
 	  NEXT_STATE(connect->state);
 	  goto cleanup;
 	}
       break;
+
+
+    case __START_REDIAL_PAUSE:
+      SET_TIMEOUT(connect->timeout, REDIAL_PAUSE_TIMEOUT);
+      connect->dial_state = __END_REDIAL_PAUSE;
+      break;
+
+
+    case __END_REDIAL_PAUSE:
+      if (TIMEOUT(connect->timeout))
+	{
+	  connect->dial_state = __CALL;
+	  goto cleanup;
+	}
+      break;
     }
 
  cleanup:
@@ -795,7 +782,7 @@ state_run_script(Connect *connect)
   /* check for carrier detect */
   if (!modem_carrier(connect->modem))
     {
-      CALLBACK(connect, PPP_ERROR, __messages[_MSG_NO_CARRIOR]);
+      CALLBACK(connect, PPP_ERROR, msg(MSG_NO_CARRIOR));
       connect->close = TRUE;
       connect->user_closed = TRUE;
       goto cleanup;
@@ -816,7 +803,7 @@ state_run_script(Connect *connect)
     case __INITALIZE:
       if (connect->account->script_list)
 	{
-	  CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_RUNNING_SCRIPT]);
+	  CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_RUNNING_SCRIPT));
 	  connect->script_entry_node = connect->account->script_list;
 	  NEXT_STATE(connect->script_state);
 	}
@@ -838,19 +825,19 @@ state_run_script(Connect *connect)
 	  break;
 
 	case SCRIPT_ENTRY_SEND:
-	  g_string_sprintf(gstr, "%s\r\n", script_entry->text->str);
+	  g_string_sprintf(gstr, "%s"NEWLINE, script_entry->text->str);
 	  modem_write(connect->modem, gstr->str, gstr->len);
 	  goto next;
 	  break;
 	  
 	case SCRIPT_ENTRY_SEND_USER:
-	  g_string_sprintf(gstr, "%s\r\n", connect->account->user->str);
+	  g_string_sprintf(gstr, "%s"NEWLINE, connect->account->user->str);
 	  modem_write(connect->modem, gstr->str, gstr->len);
 	  goto next;
 	  break;
 	  
 	case SCRIPT_ENTRY_SEND_PASSWORD:
-	  g_string_sprintf(gstr, "%s\r\n", connect->account->passwd->str);
+	  g_string_sprintf(gstr, "%s"NEWLINE, connect->account->passwd->str);
 	  modem_write(connect->modem, gstr->str, gstr->len);
 	  goto next;
 	  break;
@@ -896,7 +883,7 @@ state_run_script(Connect *connect)
   connect->script_entry_node = connect->script_entry_node->next;
   if (!connect->script_entry_node)
     {
-      CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_SCRIPT_COMPLETE]);
+      CALLBACK(connect, PPP_IN_PROGRESS, msg(MSG_SCRIPT_COMPLETE));
       NEXT_STATE(connect->state);
     }
 
@@ -959,7 +946,8 @@ state_exec_pppd(Connect *connect)
     case __DETECT_PPP_CONNECTION:
       if (pppd_device_number(connect->account, &device_number))
 	{
-	  CALLBACK(connect, PPP_CONNECTED, __messages[_MSG_PPP_CONNECTED]);
+	  connect->ppp_device = device_number;
+	  CALLBACK(connect, PPP_CONNECTED, msg(MSG_PPP_CONNECTED));
 
 	  /* run connect program */
 	  if (connect->account->connect_program->len > 0)
@@ -977,7 +965,7 @@ state_exec_pppd(Connect *connect)
 	    case PPPD_PENDING:
 	      if (TIMEOUT(connect->timeout))
 		{
-		  CALLBACK(connect, PPP_ERROR, __messages[_MSG_PPP_FAIL]);
+		  CALLBACK(connect, PPP_ERROR, msg(MSG_PPP_FAIL));
 		  connect->close = TRUE;
 		  connect->user_closed = TRUE;
 		  goto error;
@@ -1034,7 +1022,7 @@ destroy_connect(Connect *connect)
   CALLBACK(connect, PPP_DISCONNECTED, NULL);
 
   /* remove connection from hash */
-  g_hash_table_remove(__connect_hash, connect->account);
+  g_hash_table_remove(__connect_hash, connect->orig_account);
   free_connect(connect);
 }
 
