@@ -18,15 +18,18 @@
  */
 
 /* external declarations for initilization functions  */
+extern void __init_sighndl();
 extern void __init_connect();
 extern void __init_ppp();
 extern void __init_account();
 
+
 void
 gnomeppplib_init()
 {
-  __init_connect();
+  __init_sighndl();
   __init_ppp();
+  __init_connect();
   __init_account();
 }
 
