@@ -39,6 +39,6 @@ int      modem_read(Modem *modem, char *buff, int len);
 int      modem_read_ready(Modem *modem);
 void     modem_drain(Modem *modem);
 void     modem_set_speed(Modem *modem, int baud);
-int      modem_carrior(Modem *modem);
+int      modem_carrier(Modem *modem);
 
 #endif /* __modem_h__ */
