@@ -19,6 +19,10 @@
 #ifndef __pppd_h__
 #define __pppd_h__
 
+#include <gnome.h>
+#include <signal.h>
+#include "gnome-ppp.h"
+
 typedef void (*PPPCBFunc)(gpointer data);
 
 enum
@@ -40,5 +44,6 @@ gint          pppd_end_chat(Account *account);
 gint          pppd_device_number(Account *account, int *device_number);
 pid_t         pppd_pid(Account *account);
 gint          pppd_kill(Account *account);
+void          pppd_sigchld(pid_t pid, int status);
 
 #endif /* __pppd_h__ */
